module sicot_mod
contains
subroutine sicot(f, m, h, x0)

! *******************************************************************
! *                                                                 *
! * integral transform g(y, x0) of a real function f(x) defined by  *
! *                                                                 *
! * g(x0, y) = integral of f(x)/tanh(x/x0)*(1-cos(x*y)) dx          *
! *                                                                 *
! * f(x) is tabulated on the mesh of points xj = (j-1/2)*h ,        *
! * j = 1, 2, ..., n ,  with n = 2**m                               *
! * g(x0, y) is computed on the mesh of points yk = k*2pi/(n*h) ,   *
! * k = 0, 1, 2, ..., n-1                                           *
! *                                                                 *
! * input ...                                                       *
! * f(j) contains the value of f(xj) , j = 1, 2, ..., n             *
! * m is such that n = 2**iabs(m)                                   *
! * h is the step size (must be positive)                           *
! *                                                                 *
! * output ...                                                      *
! * f(k+1) contains g(x0, yk), k = 0, 1, ..., n-1                   *
! *                                                                 *
! * computing remarks ...                                           *
! * f is a real array with dimension n or more                      *
! * it is supposed that f(x) is zero for x > n*h                    *
! * external reference : sintr (sine transform of a real function). *
! *                                                                 *
! *******************************************************************

  use sintr_mod

  implicit none

  double precision, intent(in out) :: f(*)
  integer, intent(in) :: m
  double precision, intent(in) :: h
  double precision, intent(in) :: x0

  double precision :: a, b, c, d, e, s, sa
  integer :: i, j, msign, n

  logical :: debug

  debug = .false.
  if (m == 0) then
    f(1) = 0.0d0
    return
  endif

  if (h <= 0.0d0) then
    if (debug) write(*,*) ' *** incorrect step size in <sicot>, h =', h, ' ***'
    stop
  endif

  if (x0 < 0.0d0) then
    if (debug) write(*,*) ' *** incorrect input in <sicot>, x0 =', x0, ' ***'
    stop
  endif

  n = 2**iabs(m)

! *** evaluate the integral from xj to infinity of f(x)/tanh(x/x0) dx
! and store the result in f(j)

  c = 0.0d0
  if (x0 > h / 16) then
    c = dexp(-h / x0)
  endif
  s = 1.0d0 - c
  e = c**2
  a = 0.25d0 * h
  b = 0.25d0 * x0
  do i = 1, n
    if (i < n) then
      f(i) = f(i) + f(i+1)
    endif
    d = a
    if (s /= 1.0d0) then
      sa = s
      c = c * e
      s = 1.0d0 - c
      d = d + b * dlog(s / sa)
    endif
    f(i) = f(i) * d
  enddo

  j = n
  do i = 2, n
    j = j - 1
    f(j) = f(j) + f(j + 1)
  enddo
! alternative, but not yet tested:
!  do j = n - 1, 1, -1
!    f(j) = f(j) + f(j + 1)
!  enddo
  msign = -iabs(m)
  call sintr(f, msign, h)
  return
end subroutine sicot
end module sicot_mod
